"""\
$id$

This file is an installation script for this skin.  
It is meant to be used as an External Method.  

Please go through and look for comments beginning with "CHANGE", and 
change values where required.

To use, either use Plone 2.0+ to install, or add an external 
method to the root of the Plone Site that you want the skin registered 
in with the configuration:

 id: install_skin
 title: Install Skin *optional*
 module name: MySkin.Install
 function name: install

Then go to the management screen for the newly added external method
and click the 'Try it' tab. 
"""

from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFCore.utils import getToolByName
from Products.CMFFormController.FormAction import FormActionKey
from Products.Archetypes.Extensions.utils import installTypes
from Products.Archetypes.Extensions.utils import install_subskin
from Products.Archetypes.public import listTypes

from Products.HTMLProxy.config import *

from cStringIO import StringIO
import string
#from Products.CMFDynamicViewFTI.migrate import migrateFTIs

# CHANGE this to use the name of your product - this will be used to ensure
#  we register a skin directory in the correct context
from Products.HTMLProxy import skin_globals

# CHANGE this to the list of skin folders to install in the skin - these
#  will be subfolders of the skins/ folder in your product's folder.
skinFolders = ('HTMLProxy',)

# CHANGE this to the name of the new skin - this will be what the user sees
#  when choosing skins, and will be the name of a property in portal_skins.
skinName = 'HTMLProxy'

# CHANGE this to the name of the skin to base the new skin on - we will copy
#  this skin and insert the folders in skinFolders underneath 'custom' in this
#  skin's layers, and then create a new skin with the name skinName.
baseSkin = 'Plone Default'

def setupSkin (self, out):
    skinsTool = getToolByName (self, 'portal_skins')
    
    # Add directory views

    addDirectoryViews (skinsTool, 'skins', skin_globals)
    out.write ("Added directory views to portal_skins.\n")
       
    # Only add if the skin doesn't already exist
    if skinName not in skinsTool.getSkinSelections ():

        # Get the skin paths of the base skin and convert to an array
        path = skinsTool.getSkinPath (baseSkin)
        path = map (string.strip, string.split (path,','))
        
        # Add our own skin folders to this path, under 'custom'
        for skinFolder in skinFolders:
            if skinFolder not in path:
                try: 
                    path.insert (path.index ('custom') + 1, skinFolder)
                except ValueError:
                    path.append (skinFolder)
    
        # Add our skin
        path = string.join (path, ', ')    
        skinsTool.addSkinSelection (skinName, path)
    else:
        out.write ('Skin already exists, leaving it alone')
       

def removeSkin (self, out):

    skinsTool = getToolByName (self, 'portal_skins')
    
    # Delete the property which holds the skin
    if skinName in skinsTool.getSkinSelections ():
        out.write ("delete skin (skinTool) - %s" % skinName)
        #out.write (skinsTool.getSkinSelections ())
        skinsTool.manage_skinLayers (del_skin = 1, chosen = (skinName,))
       
def install (self):
    out = StringIO ()

    print >> out, "Installing HTMLProxy"

    # Install types
    classes = listTypes(PROJECTNAME)
    out.write('test -------------------------------------------\n')
    print >> out, classes
    #classes = ('PRBook', )
    installTypes(self, out, classes, PROJECTNAME)
    print >> out, "Installed types"
    out.write('test -------------------------------------------\n')

    # Install skin
#    setupSkin(self, out)
    install_subskin(self, out, GLOBALS)
    print >> out, "Installed skin"

    # Migrate FTI, to make sure we get the necessary infrastructure for the
    # 'display' menu to work.
    #migrated = migrateFTIs(self, product=PROJECTNAME)
    #print >>out, "Switched to DynamicViewFTI: %s" % ', '.join(migrated)

    portal = getToolByName(self,'portal_url').getPortalObject()

    if not hasattr(self, 'htmlproxy_tool'):
        addTool = portal.manage_addProduct['HTMLProxy'].manage_addTool
        addTool(type='HTMLProxy tool')

    # Enable portal_factory
    factory = getToolByName(self, 'portal_factory')
    types = factory.getFactoryTypes().keys()
    for add_type in ['HTMLDocument', ]:
        if add_type not in types:
            types.append(add_type)
            factory.manage_setPortalFactoryTypes(listOfTypeIds = types)

    print >> out, "Added HTMLProxy to portal_factory"

    return out.getvalue()

def uninstall (self):
    out = StringIO ()

    try:
        cptool = getToolByName(self, 'portal_controlpanel')
        cptool.unregisterApplication(PROJECTNAME)
    except:
        pass

    out.write ('Uninstallation completed.\n')
    return out.getvalue()
