# -*- coding: utf-8 -*-
#
# HTMLDocument: HTMLProxy HTMLDocument class
#                  HTMLProxy System

from Products.Archetypes.public import *
from Products.ATContentTypes.content.document import ATDocument, ATDocumentSchema
from Products.ATContentTypes.content.folder import ATFolder, ATFolderSchema
from AccessControl import ClassSecurityInfo
from Products.CMFCore import DirectoryView
from Products.CMFCore.permissions import View

from Acquisition import aq_parent
from Acquisition import aq_inner
from Acquisition import aq_base

from Products.CMFPlone.interfaces.NonStructuralFolder import INonStructuralFolder
from Products.CMFCore.utils import getToolByName

from PWURLopener import PWURLopener

import Globals
import urllib
import urllib2
import base64
import re
import cgi
import string
import urlparse
import Cookie
from zLOG import LOG,INFO

from Products.HTMLProxy.config import *

HTMLDocumentSchema = ATDocumentSchema.copy() + Schema((
        TextField("baseurl",
                  required                = True,
                  searchable              = True,
                  primary                 = False,
                  allowable_content_types = ('text/plain'),
                  default_content_type    = 'text/plain',
                  default_output_type     = 'text/plain',
                  default                 = '',
                  widget                  = StringWidget(
                        i18n_domain       = "htmlproxy",
                        label             = "Default URL",
                        label_msgid       = "hp_baseurl_label",
			maxlength         = 1000,
			size              = 60
                        )
                  ),
	LinesField("accessible",
		  required                = True,
		  searchable              = False,
		  primary                 = False,
		  allowable_content_types = ('text/plain'),
		  default_content_type    = 'text/plain',
		  default_output_type     = 'text/plain',
		  widget                  = LinesWidget(
		        i18n_domain       = "htmlproxy",
			label             = "Accessible sites",
			label_msgid       = "hp_accessible_label",
			description       = "List accessible sites. NOT recommended to set only as \'*\' for the surity reasons. <br>Specify them by IP-Addresses to prevent DNS Poizoning. <br>Example: *.foo.org 192.168.1.0/24. ",
			description_msgid = "xmldoc_noproxy_desc",
			rows              = 4,
#			cols              = 60
			)
		  ),
))

HTMLDocumentSchema["title"].widget.visible = {'edit' : 'visible'}
HTMLDocumentSchema["title"].required = True
HTMLDocumentSchema["text"].widget.visible = {'edit' : 'hidden'}
HTMLDocumentSchema["text"].required = False
HTMLDocumentSchema["allowDiscussion"].widget.visible = {'edit' : 'hidden'}
HTMLDocumentSchema["allowDiscussion"].required = False
HTMLDocumentSchema['description'].widget.visible =  {'edit' : 'visible'}
HTMLDocumentSchema['description'].required = False

HTMLDocumentSchema.moveField("relatedItems", after="accessible")

class HTMLDocument(ATDocument):
    """ HTMLProxy document class """
    schema = HTMLDocumentSchema

    # Standard security settings
    security = ClassSecurityInfo()

    Portal_type = meta_type = 'HTMLProxy'
    archetype_name = 'HTMLProxy'
    typeDescription = 'This displays an external site as a page. This acquires the site specified with baseurl etc. and displays it.'
    typeDescMsgId = 'hp_desc_msgid'

    default_output_type = 'external'
    
    allowed_content_types = []
#    global_allow = 0
    filter_content_types = 0

    security.declarePublic('canSetDefaultPage')
    def canSetDefaultPage(self):
	""" make not to be able to define the default view """
	return False

    security.declarePrivate('my_cookiesafe_b64encode')
    def my_cookiesafe_b64encode(self, data):
      try:
        ret = base64.encodestring(data)
      except:
        ret = ''
      ret = string.replace(ret, '\n', '')
      ret = string.replace(ret, '=', '_')
      return ret

    security.declarePrivate('my_cookiesafe_b64decode')
    def my_cookiesafe_b64decode(self, data):
      ret = string.replace(data, '_', '=')
      try:
        ret = base64.decodestring(ret)
      except:
        ret = ''
      return ret

    security.declarePrivate('my_urlsafe_b64encode')
    def my_urlsafe_b64encode(self, data):
      try:
        ret = base64.encodestring(data)
      except:
        ret = ''
      ret = string.replace(ret, '\n', '')
      ret = string.replace(ret, '+', '-')
      ret = string.replace(ret, '/', '_')
      return ret

    security.declarePrivate('my_urlsafe_b64decode')
    def my_urlsafe_b64decode(self, data):
      ret = string.replace(data, '_', '/')
      ret = string.replace(ret, '-', '+')
      try:
        ret = base64.decodestring(ret)
      except:
        ret = ''
      return ret

    security.declareProtected(View, 'getContent')
    def getContent(self, container, here):
        # take out query string
        # linkurl    : value of href or src attributes
        # currenturl : current url
        request = container.REQUEST
        query = request.QUERY_STRING

        try:
            linkurl = cgi.parse_qs(query)['linkurl'][0]
            linkurl = self.my_urlsafe_b64decode(linkurl)
            # add remaining query string
            rest = re.sub(r'&?currenturl=[^&]*(.*)', r'\1', query)
            rest = re.sub(r'&?linkurl=[^&]*(.*)', r'\1', rest)
            rest = re.sub(r'&?attrtype=[^&]*(.*)', r'\1', rest)
            rest = re.sub(r'^&(.*)', r'\1', rest)
            if rest:
              if urlparse.urlparse(linkurl)[4]:
                linkurl += '&' + rest
              else:
                linkurl += '?' + rest

        except:
            linkurl = self.getBaseurl()

        try:
            currenturl = cgi.parse_qs(query)['currenturl'][0]
            currenturl = self.my_urlsafe_b64decode(currenturl)
        except:
            currenturl = self.getBaseurl()

        try:
            if cgi.parse_qs(query).has_key('attrtype'):
                attrtype = cgi.parse_qs(query)['attrtype'][0]
            else:
                attrtype = 'href'
        except:
            attrtype = 'href'

        # obtain targeturl from linkurl and currenturl
        targeturl = urlparse.urljoin(currenturl, linkurl)

        # take out present objecturl
        absolute_url = here.absolute_url()

        # refer to contents of an external site
        htmlproxy_tool = getToolByName(self, 'htmlproxy_tool')
        ac = htmlproxy_tool.getAccessibleSites()
        useproxy = htmlproxy_tool.getUseHTTPProxy()
        proxy = htmlproxy_tool.getHTTPProxy()
        noproxy = htmlproxy_tool.getNoHTTPProxyFor()

	try:
          url = string.replace(targeturl, '&amp;', '&')
          url = string.replace(url, ' ', '%20')
          url_t = list(urlparse.urlparse(url))
          url_t[1] = re.sub(r':.*$', '', url_t[1])
          
          if request.has_key('HTTP_USER_AGENT'):
            agent = request['HTTP_USER_AGENT']
            
          if useproxy and self.useProxy(url, noproxy):
            proxies = {'http': proxy, 'https': proxy}
            opener = PWURLopener(agent=agent, proxies=proxies)
          else:
            opener = PWURLopener(agent=agent)

          # decode cookie
          if request.has_key('HTTP_COOKIE'):
            clientC = Cookie.SmartCookie()
            clientC.load(request['HTTP_COOKIE'])
            for NEWNAME in clientC:
              if re.match('HTMLProxy_', NEWNAME):
                encoded = re.sub(r'^HTMLProxy_(.*)', r'\1', NEWNAME)
                cookie = self.my_cookiesafe_b64decode(encoded)
                if cookie == '':
                  continue
                name = re.sub(r'(^[^;]*);.*$', r'\1', cookie)
                secure = ''
                cookie_t = re.split(';', cookie)
                for c in cookie_t:
                  if re.match('Domain=', c):
                    domain = re.sub(r'^Domain=(.*)$', r'\1', c)
                    continue
                  if re.match('Path=', c):
                    path = re.sub(r'^Path=(.*)$', r'\1', c)
                    continue
                  if re.match(r'Secure$', c):
                    secure = 'Secure'
                    continue
                if (secure == 'Secure') and (url_t[0] != 'https'):
                  continue
                if domain == url_t[1]:
                  if re.match(path, url_t[2]):
                    cookie = name + '=' + clientC[NEWNAME].value + ';'
                    opener.addheader('Cookie', cookie)

          accessible = [ac, self.getAccessible()]
          opener.accessible = accessible
          if request.REQUEST_METHOD == 'POST':
            file = opener.open(url, urllib.urlencode(request))
          else:
            file = opener.open(url)

          # convert targeturl from 'user:pass@location...' into 'location...'
          targeturl_t = list(urlparse.urlparse(file.geturl()))
          targeturl_t[1] = re.sub(r'^.*@', '', targeturl_t[1])
          targeturl = urlparse.urlunparse(targeturl_t)

          # strip query
          targeturl_t[4] = targeturl_t[5] = ''
          sq_baseurl = urlparse.urlunparse(targeturl_t)

          # filter Content-Type
	  type = file.info().getheader('Content-Type')
          if type.find('application') > -1:
            filename = re.sub(r'^.*\/', '', sq_baseurl)
            type = type + "; name=\"" + filename + "\""
            disposition = "\"\"; filename=\"" + filename + "\""
            request.RESPONSE.setHeader('Content-Disposition', disposition)
          request.RESPONSE.setHeader('Content-Type', type)

          # encode Cookie
          val = file.info().getheader('Set-Cookie')
          if val:
            serverC = Cookie.SmartCookie()
            serverC.load(val)
            for NAME in serverC:
              # set domain
              if serverC[NAME]['domain']:
                domain = 'Domain=' + serverC[NAME]['domain']
              else:
                host = re.sub(r':.*$', '', targeturl_t[1])
                domain = 'Domain=' + host
              # set path
              if serverC[NAME]['path']:
                path = 'Path=' + serverC[NAME]['path']
              else:
                path = urlparse.urlparse(targeturl)[2]
                path = re.sub(r'([^\/]*\/).*$', r'\1', path)
                path = 'Path=' + path
              # set secure
              if serverC[NAME]['secure']:
                secure = 'Secure;'
              else:
                secure = ''
              # set expires
              if serverC[NAME]['expires']:
                expires = 'expires=' + serverC[NAME]['expires']
              else:
                expires = ''
              # set NEWNAME
              concat = NAME + ';' + domain + ';' + path + ';' + secure
              NEWNAME = self.my_cookiesafe_b64encode(concat)
              cookie = 'HTMLProxy_' + NEWNAME + '="' + serverC[NAME].value + '";' + expires
              # set cookie
              request.RESPONSE.setHeader('Set-Cookie', cookie)

          # pass Content-Disposition
          val = file.info().getheader('Content-Disposition')
          if val:
            request.RESPONSE.setHeader('Content-Disposition', val)

	  content = file.read()
	  file.close()
        except 'notaccessible':
          content = '<h3>Accesse NOT permitted</h3>' + targeturl
          return {'data': content, 'type': 'text/html', 'attrtype': attrtype}
        except:
          content = '<h3>Specified URL was wrong or the error occurred.</h3>' + targeturl
          return {'data': content, 'type': 'text/html', 'attrtype': attrtype}

        if type.find('text/html') > -1:
          content = self.my_encode(content, 'utf8')
          mb1 = re.search(r'<base[^>]+href\s*=\s*[\'"]([^\'">]+)[\'"]\s*', content)
          if mb1:
            sq_baseurl = mb1.group(1)
          else:
            mb2 = re.search(r'<base[^>]+href\s*=\s*([^\'">]+)\s*', content)
            if mb2:
              sq_baseurl = mb2.group(1)
          content = re.sub(r'<base[^>]*>', '', content)
          content = self.rewrite_attr(content, sq_baseurl, ['href', 'HREF', 'src', 'SRC', 'action', 'ACTION'])
          content = self.add_hidden(content)

	return {'data': content, 'type': type, 'attrtype': attrtype}

    security.declarePrivate('my_encode')
    def my_encode(self, data, code):
      """ encode data """
      encodings = ('utf-8', 'ascii', 'euc-jp', 'sjis', 'iso-8859-1', 'utf-16-le', 'utf-16-be')

      for i in encodings:
        try:
          return unicode(data, i).encode(code)
        except:
          pass
      return data

    security.declarePrivate('rewrite_attr')
    def rewrite_attr(self, data, baseurl, attrs):
      """ rewrite URL of the specified attributes(list) """
      baseurl = self.my_urlsafe_b64encode(baseurl)

      def attrrepl(match):
        matches = match.groups()
        if re.match('mailto:', matches[1]):
          ret = matches[0] + '"' + matches[1] + '"' + matches[2]
          return ret
        linkurl = self.my_urlsafe_b64encode(matches[1])
        ret = matches[0] + '"?currenturl=' + baseurl + '&linkurl=' + linkurl + '&attrtype=' + attr + '"' + matches[2]
        return ret

      #for attr in attrs:
          #data = re.sub(r'(' + attr + r'\s*=\s*)[\'"]([^\'">]+)[\'"](\s*)', attrrepl, data)
          #data = re.sub(r'(' + attr + r'\s*=\s*)([^\'">\s]+)(\s*)', attrrepl, data)
      return data 

    security.declarePrivate('add_hidden')
    def add_hidden(self, data):
      """
      Pick up the query string in <form action=> and add as <input type=hidden>.
      This processing corresponds to the phenomenon that 
        query in action attribute is disregarded when method=GET.
      It is necessary to turn on the data processed by rewrite_attr method.
      """
      for attr in ('action', 'ACTION'):
        data = re.sub(r'(' + attr + r'\s*=\s*)"\?currenturl=([^&]*)&linkurl=([^&]*)&attrtype=([^"]*)"([^>]*>)', r'\1"?currenturl=\2&linkurl=\3&attrtype=\4"\5<input type="hidden" name="currenturl" value="\2"><input type="hidden" name="linkurl" value="\3"><input type="hidden" name="attrtype" value="\4">', data)
      return data

    security.declarePrivate('useProxy')
    def useProxy(self, url, direct):
      """ This method judges whether to use proxy """
      host = re.sub(r':.*$', '', urlparse.urlparse(url)[1])
      direct = direct.split(',')

      # host : IP Address
      if re.match('\d+\.\d+\.\d+\.\d+$', host):
        el = re.split('\.', host)
        host_addr = string.atoi(el[0]) * 256**3 + string.atoi(el[1]) * 256**2 + string.atoi(el[2]) * 256 + string.atoi(el[3])
        for d in direct:
          d = d.lstrip().rstrip()
          if not d:
            continue
          # list : IP Address
          if re.match('\d+\.\d+\.\d+\.\d+(/\d+)?$', d):
            el = re.split('[\./]', d)
            addr = string.atoi(el[0]) * 256**3 + string.atoi(el[1]) * 256**2 + string.atoi(el[2]) * 256 + string.atoi(el[3])
            if len(el) > 4:
              mask = string.atoi(el[4])
              if mask > 32:
                mask = 32 
            else:
              mask = 32
            shift = 32 - mask
            if host_addr >> shift == addr >> shift:
              return 0
        return 1

      # host : Name
      else:
        for d in direct:
          d = d.lstrip().rstrip()
          if not d:
            continue
          # list : Name
          if not re.match('\d+\.\d+\.\d+\.\d+(/\d+)?$', d):
            if re.search(d + r'$', host):
              return 0
        return 1

    # enable FTP/WebDAV and friends
    PUT = ATDocument.PUT

Globals.InitializeClass(HTMLDocument)
registerType(HTMLDocument, PROJECTNAME)
